/* daemonitor/src/daemonitor/100_io.c
 *
 *  (c)2007, Laurence Withers, <l@lwithers.me.uk>.
 *  Released under the GNU GPLv3. See file COPYING or
 *  http://www.gnu.org/copyleft/gpl.html for details.
*/



/* I/O ROUTINES ***********************************************************************************
 *
 *  This file provides a small set of I/O wrapper routines, for doing things like safely dealing
 *  with interrupted writes.
 */



/* PUBLIC API ************************************************************************************/



/* safe_write_fixed()
 *  As write(2), except that it tries again in the face of partial writes or signal delivery.
 */
ssize_t safe_write_fixed(int fd, const char* buf, size_t amt);



/* safe_sleep_fixed()
 *  Sleeps for a fixed period of seconds and nanoseconds. Restarts sleep with correct time if
 *  interrupted by signal delivery.
 */
void safe_sleep_fixed(int s, int ns);



/* IMPLEMENTATION ********************************************************************************/



/* safe_write_fixed()
 *  Copied from libgslutil (GPL-3).
 */
ssize_t safe_write_fixed(int fd, const char* buf, size_t amt) __attribute__((nonnull));
ssize_t safe_write_fixed(int fd, const char* buf, size_t amt)
{
    ssize_t ret, total = 0;

    while(amt) {
        ret = write(fd, buf, amt);
        if(ret == -1) {
            if(errno == EINTR) continue;
            return -1;
        }

        total += ret;
        buf += ret;
        amt -= ret;
    }

    return total;
}



/* safe_sleep_fixed()
 *  Adapted from libgslutil (GPL-3).
 */
void safe_sleep_fixed(int s, int ns)
{
    struct timespec req, rem;

    req.tv_sec = s;
    req.tv_nsec = ns;

  restart:
    if(nanosleep(&req, &rem) == -1) {
        if(errno == EINTR) {
            memcpy(&req, &rem, sizeof(struct timespec));
            goto restart;
        }
        return;
    }
}



/* options for text editors
kate: replace-trailing-space-save true; space-indent true; tab-width 4;
vim: expandtab:ts=4:sw=4
*/
